@@ -15,6 +15,7 @@ from .job_execution import JobExecution, JobExecutionData, JobExecutionStatus
 
 JOB_TABLE = "jobs"
 JOB_EXECUTION_TABLE = "job_executions"
+SUPPORTED_PROVIDERS = ["simulator", "ibm", "rigetti", "ionq", "quera", "iqm", "custom"]
 
 class MultipleExecutionJob():
     override_job_id: Optional[str]
@@ -29,8 +30,10 @@ class MultipleExecutionJob():
 class Quave:
     """Client for interacting with the Quave quantum computing platform."""
 
-    default_backend = "simulate"
-    default_provider = "simulator"
+    default_backend = "AerSimulator"
+    default_provider = "Simulator"
+    poll_interval = 1.0  # Start with 1 second intervals
+    max_poll_interval = 5.0  # Max 5 seconds between polls
 
     def __init__(self, email: Optional[str] = None, password: Optional[str] = None):
         """
@@ -151,19 +154,18 @@ class Quave:
         Raises:
             Warning: If the backend is not supported
         """
-        provider = Quave.default_provider
         if backend == Quave.default_backend:
-            return backend, provider
+            return backend, Quave.default_provider
         provider = None
         supported_backends = self.list_backends()
         if not supported_backends:
-            warnings.warn("Failed to fetch supported backends so unable to validate backend", FutureWarning)
+            warnings.warn(f"Failed to fetch supported backends so unable to validate {backend}", FutureWarning)
         else:
             for backend_provider, backend_list in supported_backends.items():
                 if backend in backend_list:
                     provider = backend_provider
             if not provider:
-                warnings.warn("Backend not supported. Defaulting to Aer simulator", FutureWarning)
+                warnings.warn(f"Backend not supported. Defaulting to {Quave.default_backend}", FutureWarning)
                 backend = Quave.default_backend
                 provider = Quave.default_provider
         return backend, provider
@@ -212,7 +214,9 @@ class Quave:
             payload = {
                 "qasm": qasm,
                 "shots": shots,
-                "backend": backend
+                "backend": backend,
+                "queued_at": datetime.now(timezone.utc).isoformat(),
+                "provider": provider,
             }
 
             expected_executions = 1
@@ -226,6 +230,7 @@ class Quave:
             if multi_execution:
                 payload["job_id"] = multi_execution.override_job_id
                 payload["update_job"] = multi_execution.update_job_on_completion
+                payload["reset_status"] = multi_execution.reset_status
 
             headers = {"Content-Type": "application/json", "Authorization": self._get_bearer_token()}
             url = f"{self._base_url}/v1/jobs"
@@ -254,13 +259,10 @@ class Quave:
             
             if len(execution_ids) < expected_executions:
                 warnings.warn(f"Unable to parse all executions. Proceeding with {len(execution_ids)} executions", RuntimeWarning)
-
+            
             job_data = {"id": job_id, "user_id": user_id, "status": "PENDING", "qasm": qasm}
             return_job = JobData(job_data)
-            if not multi_execution or multi_execution.reset_status:
-                self._supabase.table(JOB_TABLE).upsert(job_data).execute()
-            
-            rows = []
+
             for i in range(len(execution_ids)):
                 execution_data = {
                     "id": execution_ids[i], 
@@ -273,20 +275,18 @@ class Quave:
                 }
                 if parameters:
                     execution_data["parameters"] = param_sets[i]
-
-                rows.append(execution_data)
                 return_executions.append(JobExecutionData(execution_data))
 
-            self._supabase.table(JOB_EXECUTION_TABLE).insert(rows).execute()
             return return_job, return_executions
         except FunctionsError as e:
             raise Exception(f"Failed to store circuit execution: {e.message}") from e 
         except Exception as e:
-            raise Exception(f"Failed to execute circuit: {str(e)}") from e # DEBUG - remove the str
-        
+            raise Exception(f"Failed to execute circuit: {str(e)}") from e
+    
     async def _await_job_completion(self, job: Job, timeout: float = 10000) -> Optional[JobData]:
         """
         Waits for a job to be complete and then returns JobData object asynchronously.
+        Uses a hybrid approach with realtime subscriptions and polling fallback for reliability.
 
         Args:
             job (Job): The job to await completion for.
@@ -295,36 +295,61 @@ class Quave:
         Returns:
             Optional[JobData]: Parsed row of the job if complete, otherwise None.
         """
-        await self._validate_realtime_session()
-        loop = asyncio.get_running_loop()
-        future = loop.create_future()
-
-        def on_update(payload):
-            new_data = payload.get("data")
-            if new_data and new_data.get("record").get("id") == job.job_id:
-                raw_data = new_data.get("record")
-                job_data = JobData(raw_data)
-                if job_data._cached_in_final_state() and not future.done():
-                    future.set_result(job_data)
-
-        channel = await self._realtime.channel("job_updates").on_postgres_changes(
-            "UPDATE", schema="public", table=JOB_TABLE, filter=f"id=eq.{job.job_id}", callback=on_update
-        ).subscribe()
-
-        await asyncio.sleep(0.1)  # Allow some time for the subscription to be established
-
+        start_time = asyncio.get_event_loop().time()
+        
+        # Try realtime subscription first (for fast notifications)
         try:
+            await self._validate_realtime_session()
+            loop = asyncio.get_running_loop()
+            future = loop.create_future()
+
+            def on_update(payload):
+                new_data = payload.get("data")
+                if new_data and new_data.get("record").get("id") == job.job_id:
+                    raw_data = new_data.get("record")
+                    job_data = JobData(raw_data)
+                    if job_data._cached_in_final_state() and not future.done():
+                        future.set_result(job_data)
+
+            channel = await self._realtime.channel("job_updates").on_postgres_changes(
+                "UPDATE", schema="public", table=JOB_TABLE, filter=f"id=eq.{job.job_id}", callback=on_update
+            ).subscribe()
+
+            await asyncio.sleep(0.1)  # Allow subscription to establish
+
+            # Check if already complete
             current = self._get_job(job)
             if current and current._cached_in_final_state():
-                if not future.done():
-                    future.set_result(current)
-            return await asyncio.wait_for(future, timeout=timeout)
-        except asyncio.TimeoutError:
-            warnings.warn(f"Timeout waiting for job {job.job_id}", RuntimeWarning)
-            return None
-        finally:
-            await self._realtime.remove_channel(channel)
+                await self._realtime.remove_channel(channel)
+                return current
 
+            # Wait for realtime update with shorter timeout for fallback
+            try:
+                return await asyncio.wait_for(future, timeout=min(30.0, timeout * 0.3))
+            except asyncio.TimeoutError:
+                # Realtime failed, fall back to polling
+                await self._realtime.remove_channel(channel)
+                
+        except Exception as e:
+            # Realtime setup failed, use polling only
+            warnings.warn(f"Realtime subscription failed for job {job.job_id}: {e}. Falling back to polling.", RuntimeWarning)
+        
+        poll_interval = Quave.poll_interval
+
+        while True:
+            elapsed = asyncio.get_event_loop().time() - start_time
+            if elapsed >= timeout:
+                warnings.warn(f"Timeout waiting for job {job.job_id}", RuntimeWarning)
+                return None
+                
+            current = self._get_job(job)
+            if current and current._cached_in_final_state():
+                return current
+                
+            # Exponential backoff for polling interval
+            await asyncio.sleep(poll_interval)
+            poll_interval = min(poll_interval * 1.2, Quave.max_poll_interval)
+    
     async def _await_execution_completion(self, execution: JobExecution, timeout: float = 10000) -> Optional[JobExecutionData]:
         """
         Waits for an execution to be complete and then returns JobExecutionData object asynchronously.
@@ -336,32 +361,60 @@ class Quave:
         Returns:
             Optional[JobExecutionData]: Parsed row of the execution if complete, otherwise None.
         """
-        await self._validate_realtime_session()
-        loop = asyncio.get_running_loop()
-        future = loop.create_future()
-
-        def on_update(payload):
-            new_data = payload.get("data")
-            if new_data and new_data.get("record").get("id") == execution.id:
-                raw_data = new_data.get("record")
-                execution_data = JobExecutionData(raw_data)
-                if execution_data._cached_in_final_state() and not future.done():
-                    future.set_result(execution_data)
-
-        channel = await self._realtime.channel("execution_updates").on_postgres_changes(
-            "UPDATE", schema="public", table=JOB_EXECUTION_TABLE, filter=f"id=eq.{execution.id}", callback=on_update
-        ).subscribe()
-
+        start_time = asyncio.get_event_loop().time()
+        
+        # Try realtime subscription first (for fast notifications)
         try:
+            await self._validate_realtime_session()
+            loop = asyncio.get_running_loop()
+            future = loop.create_future()
+
+            def on_update(payload):
+                new_data = payload.get("data")
+                if new_data and new_data.get("record").get("id") == execution.id:
+                    raw_data = new_data.get("record")
+                    execution_data = JobExecutionData(raw_data)
+                    if execution_data._cached_in_final_state() and not future.done():
+                        future.set_result(execution_data)
+
+            channel = await self._realtime.channel("execution_updates").on_postgres_changes(
+                "UPDATE", schema="public", table=JOB_EXECUTION_TABLE, filter=f"id=eq.{execution.id}", callback=on_update
+            ).subscribe()
+
+            await asyncio.sleep(0.1)  # Allow subscription to establish
+
+            # Check if already complete
+            current = self._get_execution(execution)
+            if current and current._cached_in_final_state():
+                await self._realtime.remove_channel(channel)
+                return current
+
+            # Wait for realtime update with shorter timeout for fallback
+            try:
+                return await asyncio.wait_for(future, timeout=min(30.0, timeout * 0.3))
+            except asyncio.TimeoutError:
+                # Realtime failed, fall back to polling
+                await self._realtime.remove_channel(channel)
+                
+        except Exception as e:
+            # Realtime setup failed, use polling only
+            warnings.warn(f"Realtime subscription failed for execution {execution.id}: {e}. Falling back to polling.", RuntimeWarning)
+        
+        poll_interval = Quave.poll_interval
+
+        while True:
+            elapsed = asyncio.get_event_loop().time() - start_time
+            if elapsed >= timeout:
+                warnings.warn(f"Timeout waiting for execution {execution.id} after {elapsed:.1f}s", RuntimeWarning)
+                return None
+                
             current = self._get_execution(execution)
             if current and current._cached_in_final_state():
                 return current
-            return await asyncio.wait_for(future, timeout=timeout)
-        except asyncio.TimeoutError:
-            warnings.warn(f"Timeout waiting for job execution {execution.id}", RuntimeWarning)
-            return None
-        finally:
-            await self._realtime.remove_channel(channel)
+                
+            # Exponential backoff for polling interval
+            await asyncio.sleep(poll_interval)
+            poll_interval = min(poll_interval * 1.2, Quave.max_poll_interval)
 
     def _get_job(self, job: Job) -> JobData:
         """
@@ -489,10 +542,16 @@ class Quave:
                 executions = self._get_job_executions(job)
                 execution_result = next((ex for ex in executions if ex.id == execution_id), None)
 
-                if not execution_result or execution_result.get_status() != JobExecutionStatus.COMPLETED.value or not execution_result.get_counts():
+                if not execution_result:
+                    warnings.warn(f"Unable to retrieve execution results for iteration {i + 1}. Terminating.", RuntimeWarning)
+                    return job
+                elif execution_result.get_status() != JobExecutionStatus.COMPLETED.value:
+                    warnings.warn(f"Execution did not complete successfully in iteration {i + 1}. Terminating.", RuntimeWarning)
+                    return job
+                elif not execution_result.get_counts():
                     warnings.warn(f"Unable to retrieve counts for iteration {i + 1}. Terminating.", RuntimeWarning)
                     return job
-                
+
                 try:
                     parameters = parameter_update_fn(execution_result.get_counts())
                     _, execution_data_list = self._execute_circuit(circuit, parameters, shots, backend, MultipleExecutionJob(override_job_id=job.job_id))
@@ -527,24 +586,27 @@ class Quave:
             return backends
         return backends.get(provider)
 
-    def get_backend_stats(self, backend_name: str, provider: str) -> Optional[dict[str, Any]]:
+    def get_backend_stats(self, backend_name: str) -> Optional[dict[str, Any]]:
         """
         Fetches statistics and configuration for a specific backend.
 
         Args:
             backend_name (str): The backend identifier.
-            provider (str): The provider of the backend.
         
         Returns:
             Optional[dict]: Backend statistics and configuration, or None if not found.
+
+        Raises:
+            RuntimeError: If the backend validation fails.
         """
-        if provider not in ["simulator", "ibm", "braket", "custom"]:
-            warnings.warn(f"Provider {provider} not supported. Defaulting to simulator")
-            provider = "simulator"
+
+        validated_backend, provider = self._validate_backend(backend_name)
+        if not provider:
+            raise RuntimeError("Failed to validate backend")
         
         self._validate_supabase_session()
         url = f"{self._base_url}/v1/backends/statistics"
-        params = {"backend": backend_name, "provider": provider}
+        params = {"backend": validated_backend, "provider": provider}
         headers = {"Content-Type": "application/json", "Authorization": self._get_bearer_token()}
         response = requests.get(url, headers=headers, params=params)
         if response.status_code != 200:
